import requests
import json
import pandas as pd
from configparser import ConfigParser
import logging
from .exceptions import QualitubeException


config = ConfigParser()
config.read("config.ini")
API_KEY = config['credentials']['api_key']


class PlaylistItems:
    """
    Wrapper class to the YouTube Data API v3's `PlaylistItems` endpoint
    with extra functionality.
    """
    def __init__(self, playlist_id, api_key=API_KEY):
        self.playlist_id = playlist_id
        self.api_key = api_key

    def _try_parse(self, item, key):
        try:
            parsed = item[key]
        except KeyError:
            logging.warn(
                f"YouTube Data API v3 does not provide the `{key}` parameter fo"
                f"r the requested playlist item. Setting it as `None`"
            )
            parsed = None
        return parsed
    
    def _parse_response(self, data):
        """Parses the API 'PlaylistItems: list' endpoint's JSON
        response for the retrieval of video metadata."""
        raw = json.loads(data)
        
        try:
            items = raw["items"]
        except KeyError:
            if "error" in raw.keys():
                raise QualitubeException(
                    f"\nAre you sure you set qualitube's config.ini file correctly?"
                    f"\nYou are getting the following error from YouTube's API response:"
                    f"\n\t{raw}"
                )
            raise

        parsed = []
        for item in items:
            parsed.append({
                'id': self._try_parse(item['contentDetails'], 'videoId'),
                'title': self._try_parse(item['snippet'], 'title'),
                'description': self._try_parse(item['snippet'], 'description'),
                'published_at': self._try_parse(item['snippet'], 'publishedAt')
            })
            logging.info(f"Got PlaylistItem -> id: {item['id']} / title: {item['snippet']['title']}")
        try:
            next_page_token = raw["nextPageToken"]
        except KeyError:
            next_page_token = False

        return parsed, next_page_token

    def get_playlist_items_data(self, page_token=None):
        """Uses the YouTube Data API v3 'PlaylistItems: list' endpoint 
        to get all videos from PlaylistItems (a youtube playlist)."""
        headers = {
            'Accept': 'application/json'
        }

        params = [
            ('part', ['contentDetails', 'snippet']),
            ('playlistId', self.playlist_id),
            ('key', self.api_key),
            ('maxResults', 50)
        ]

        if page_token:
            params.append(('pageToken', page_token))

        r = requests.get(
            'https://youtube.googleapis.com/youtube/v3/playlistItems',
            headers=headers,
            params=params
        )

        videos_data, next_page_token = self._parse_response(r.text)

        if next_page_token:
            return videos_data + self.get_playlist_items_data(page_token=next_page_token)

        return videos_data
    
    def to_df(self):
        data = self.get_playlist_items_data()
        df = pd.DataFrame(data)
        return df



from .playlist_items import PlaylistItems
from configparser import ConfigParser
from .videos import Videos
import pandas as pd
import logging
import sys

config = ConfigParser()
config.read("config.ini")
CHANNEL_IDS = config['channels']['ids'].split("\n")[1:]

def get_playlist_items_objs(channel_ids: str) -> PlaylistItems:
    """Get list of PlaylistItems objects from channel ids."""
    objs = [PlaylistItems(channel_id) for channel_id in channel_ids]
    return objs

def chunks(lst: list, n) -> list:
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i:i + n]

def main():
    logging.basicConfig(
        level=logging.INFO,
        handlers=[
            logging.FileHandler('pipeline.log'),
            logging.StreamHandler(sys.stdout)
        ]
    )

    logging.info('Beginning of pipeline.')

    playlists = get_playlist_items_objs(channel_ids=CHANNEL_IDS)
    dfs = [playlist.to_df() for playlist in playlists]
    raw = pd.concat(dfs)
    videos_ids_lst = raw['id'].to_list()
    chunked = list(chunks(videos_ids_lst, 50))

    dfs = []
    for chunk in chunked:
        dfs.append(Videos(chunk).to_df())

    df = pd.concat(dfs).reset_index(drop=True)
    df.to_csv('corpus.csv', index=False)

    logging.info('End of pipeline.')

if __name__ == "__main__":
    main()



import requests
import json
from configparser import ConfigParser

config = ConfigParser()
config.read("config.ini")
API_KEY = config['credentials']['api_key']

class Channel:
    """
    A wrapper to get all videos' data from a given YouTube channel.
    """
    def __init__(self, channel_id, api_key=API_KEY):
        self.channel_id = channel_id
        self.api_key = api_key

    def _parse_response(self, data):
        """Parses the API 'Search: list' endpoint's JSON response for
        the retrieval of video metadata."""
        raw = json.loads(data)
        items = raw["items"]

        parsed = []
        for item in items:
            parsed.append({
                'id': item['id']['videoId'],
                'title': item['snippet']['title'],
                'description': item['snippet']['description'],
                'published_at': item['snippet']['publishedAt']
            })
        try:
            next_page_token = raw["nextPageToken"]
        except KeyError:
            next_page_token = False

        return parsed, next_page_token

    def get_videos_data(self, page_token=None):
        """Uses the YouTube Data API v3 'Search: list' endpoint to get all
        videos from the Channel."""
        headers = {
            'Accept': 'application/json'
        }

        params = [
            ('part', 'snippet'),
            ('channelId', self.channel_id),
            ('key', self.api_key),
            ('maxResults', 50),
            ('type', 'video')
        ]

        if page_token:
            params.append(('pageToken', page_token))

        r = requests.get(
            'https://youtube.googleapis.com/youtube/v3/search',
            headers=headers,
            params=params
        )

        videos_data, next_page_token = self._parse_response(r.text)

        if next_page_token:
            return videos_data + self.get_videos_data(page_token=next_page_token)

        return videos_data



import requests
import json
import pandas as pd
from configparser import ConfigParser
import logging
from .exceptions import QualitubeException


config = ConfigParser()
config.read("config.ini")
API_KEY = config['credentials']['api_key']


class Videos:
    """
    Wrapper Class to the YouTube Data API v3's `Videos` endpoint with
    extra functionalities.
    """
    def __init__(self, videos_ids, api_key=API_KEY):
        self.videos_ids = videos_ids
        self.api_key = api_key
    
    def _try_parse(self, item, key):
        try:
            parsed = item[key]
        except KeyError:
            logging.warn(
                f"YouTube Data API v3 does not provide the `{key}` parameter fo"
                f"r the requested video. Setting it as `None`"
            )
            parsed = None
        return parsed

    def _parse_response(self, data):
        raw = json.loads(data)
        try:
            items = raw['items']
        except KeyError:
            if "error" in raw.keys():
                raise QualitubeException(
                    f"\nAre you sure you set qualitube's config.ini file correctly?"
                    f"\nYou are getting the following error from YouTube's API response:"
                    f"\n\t{raw}"
                )
            raise

        parsed = []
        for item in items:
            parsed.append({
                'channel_id': self._try_parse(item['snippet'], 'channelId'),
                'channel_title': self._try_parse(item['snippet'], 'channelTitle'),
                'video_id': self._try_parse(item, 'id'),
                'video_title': self._try_parse(item['snippet'], 'title'),
                'video_description': self._try_parse(item['snippet'], 'description'),
                'video_tags': self._try_parse(item['snippet'], 'tags'),
                'video_published_at': self._try_parse(item['snippet'], 'publishedAt'),
                'video_view_count': self._try_parse(item['statistics'], 'viewCount'),
                'video_like_count': self._try_parse(item['statistics'], 'likeCount'),
                'video_dislike_count': self._try_parse(item['statistics'], 'dislikeCount'),
                'video_favorite_count': self._try_parse(item['statistics'], 'favoriteCount'),
                'video_comment_count': self._try_parse(item['statistics'], 'commentCount')
            })
            logging.info(f"Got Video -> id: {item['id']} title: {item['snippet']['title']}")

        try:
            next_page_token = raw["nextPageToken"]
        except KeyError:
            next_page_token = False
        
        return parsed, next_page_token

    def _get_ids_parameter(self):
        ids = ','.join(self.videos_ids)
        return ids

    def get_data(self, page_token=None):
        headers = {
            'Accept': 'application/json',
        }

        params = [
            ('part', 'snippet,statistics'),
            ('id', self._get_ids_parameter()),
            ('key', self.api_key)
        ]

        if page_token:
            params.append(
                ('pageToken', page_token)
            )

        r = requests.get('https://youtube.googleapis.com/youtube/v3/videos', headers=headers, params=params)
        
        videos_data, next_page_token = self._parse_response(r.text)

        if next_page_token:
            return videos_data + self.get_data(next_page_token)

        return videos_data
    
    def to_df(self):
        df = pd.DataFrame(self.get_data())
        return df
